@@ -1,4 +1,6 @@
 # coding=utf-8
+import os
+
 __author__ = "Dmitry Zhiltsov"
 __copyright__ = "Copyright 2013, Dmitry Zhiltsov"
 __license__ = "BSD "
@@ -28,6 +30,7 @@ FIELDS = (
     'sn', # 'last_name',
     'middleName', # 'middle_name',
     'description', # 'full_name',
+    'memberOf',
 
     'company', # 'company',
     'department', # 'department',
@@ -65,12 +68,15 @@ class LDAP(object):
 
     def init_app(self, app):
         app.config.setdefault('LDAP_HOST', '127.0.0.1')
+        app.config.setdefault('LDAP_PORT', 389)
+        app.config.setdefault('LDAP_SCHEMA', 'ldap')
         app.config.setdefault('LDAP_DOMAIN', 'example.com')
         app.config.setdefault('LDAP_LOGIN_VIEW', 'login')
         app.config.setdefault('LDAP_SEARCH_BASE', 'OU=Users,DC=example,DC=com')
         app.config.setdefault('LDAP_LOGIN_TEMPLATE', 'login.html')
         app.config.setdefault('LDAP_SUCCESS_REDIRECT', 'index')
         app.config.setdefault('LDAP_PROFILE_KEY', 'sAMAccountName')
+        app.config.setdefault('LDAP_AVATAR_LOC', None)
         # Use the newstyle teardown_appcontext if it's available,
         # otherwise fall back to the request context
         if hasattr(app, 'teardown_appcontext'):
@@ -82,7 +88,10 @@ class LDAP(object):
 
     def connect(self):
         print self.app.config['LDAP_HOST']
-        self.conn = ldap.init(self.app.config['LDAP_HOST'])
+        self.conn = ldap.initialize('{0}://{1}:{2}'.format(
+            self.app.config['LDAP_SCHEMA'],
+            self.app.config['LDAP_HOST'],
+            self.app.config['LDAP_PORT']))
         return self.conn
 
     def teardown(self, exception):
@@ -94,6 +103,12 @@ class LDAP(object):
         fields = list(FIELDS)
         if fields and not 'sAMAccountName' in fields:
             fields.append('sAMAccountName')
+        if fields and self.app.config['LDAP_AVATAR_LOC']:
+            fields.append('jpegPhoto')
+            if 'LDAP_PROFILE_KEY' in self.app.config:
+                pass
+            else:
+                fields.append('userPrincipalName')
         records = self.conn.search_s(self.app.config['LDAP_SEARCH_BASE'],
                                     ldap.SCOPE_SUBTREE, query, fields)
         # except ldap.LDAPError as e:
@@ -109,6 +124,15 @@ class LDAP(object):
                 except:
                     newrec[field] = None
             res.append(newrec)
+        if 'jpegPhoto' in res[0]:
+            self.jpegPhoto = res[0]['jpegPhoto']
+            if 'LDAP_PROFILE_KEY' in self.app.config:
+                self.userPrincipalName = res[0][self.app.config['LDAP_PROFILE_KEY']]
+            else:
+                self.userPrincipalName = res[0]['userPrincipalName']
+        else:
+            self.jpegPhoto = None
+            self.userPrincipalName = None
         return res
 
     def ldap_login(self, username, pwd):
@@ -121,6 +145,7 @@ class LDAP(object):
             for user in self.ldap_query(query):
                 mu = dict(_id=user[self.app.config['LDAP_PROFILE_KEY']])
                 mu['username'] = user.get('sAMAccountName')
+                mu['memberOf'] = user.get('memberOf')
                 mu['ad_cn'] = user.get('distinguishedName')
                 mu['fired'] = 'OU=Retired' in mu['ad_cn']
                 mu['staff'] = True
@@ -143,14 +168,29 @@ class LDAP(object):
                 mu['mail'] = user.get('mail')
                 mu['location'] = user.get('streetAddress')
                 mu['ip_phone'] = user.get('ipPhone')
+
                 if self.mongo is not None:
                     self.mongo.db.users.save(mu), mu['full_name']
                 self.mu = mu
+                self.save_avatar()
+
+            if self.app.config['LDAP_REQUIRED_GROUP'] is not None:
+                if self.app.config['LDAP_REQUIRED_GROUP'] not in self.mu['memberOf']:
+                    flash(u"Login successful but not authorized.")
+                    return False
 
             return True
 
         except ldap.LDAPError as e:
-                return self.ldap_err(e)
+            return self.ldap_err(e)
+        except Exception as e:
+            return self.other_err(e)
+
+    def save_avatar(self):
+        if self.app.config['LDAP_AVATAR_LOC']:
+            avatar_file = open(os.path.join(self.app.config['LDAP_AVATAR_LOC'],self.userPrincipalName+'.jpg'), 'wb')
+            avatar_file.write(self.jpegPhoto)
+            avatar_file.close()
 
     def login(self):
         """
@@ -174,6 +214,10 @@ class LDAP(object):
         flash(message=dict(exc.message)['desc'], category='error')
         return False
 
+    def other_err(self, exc):
+        flash(message=exc.message, category='error')
+        return False
+
     @property
     def connection(self):
         ctx = stack.top
@@ -202,10 +246,10 @@ def short_name(user):
     gn = gn or fn[-2] if len(fn) > 2 else fn[-1]
 
     if gn:
-        sn.append(u'{}.'.format(gn[0]))
+        sn.append(u'{0}.'.format(gn[0]))
     mn = mn or fn[-1] if len(fn) > 2 else ''
 
     if mn:
-        sn.append(u'{}.'.format(mn[0]))
+        sn.append(u'{0}.'.format(mn[0]))
 
     return u' '.join(sn).encode('utf8')
